import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/utils/ifps_dialog_utils.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';

enum UploadType {
  ///图片
  TYPE_IMAGE,

  ///文件
  TYPE_FILE,
}

///
/// 选择图片上传
///
class FileSelectView extends StatefulWidget {
  final UploadType uploadType;
  final UploadMode uploadMode;
  final ValueChanged<List<UploadResultEntity>>? callback;

  String dirId;
  String dirName;

  FileSelectView(
      {Key? key,
      required this.dirId,
      required this.dirName,

      this.uploadType = UploadType.TYPE_IMAGE,
      this.callback,
      this.uploadMode = UploadMode.MULTIPLE})
      : super(key: key);

  @override
  FileSelectViewState createState() => FileSelectViewState();
}

class FileSelectViewState extends State<FileSelectView> {
  ///上传的文件夹名字
  String _folderName = '';

  ///上传的文件夹id
  String _folderId = '';

  List<UploadResultEntity> _list = [];

  @override
  Widget build(BuildContext context) {
    double contentHeight = MediaQuery.of(context).size.height * 0.9;
    return Material(
        type: MaterialType.transparency,
        child: Column(children: <Widget>[
          Expanded(
            child: GestureDetector(
              behavior: HitTestBehavior.translucent,
              onTap: () {
                Navigator.of(context).pop();
              },
              child: Container(),
            ),
          ),
          Container(
            height: contentHeight,
            decoration: const BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(10.0),
                  topRight: Radius.circular(10.0)),
            ),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _buildUploadTitle(),
                const SizedBox(
                  height: 20,
                ),
                InkWell(
                  child: Container(
                    height: 40,
                    alignment: Alignment.center,
                    margin: const EdgeInsets.only(left: 15, right: 15),
                    padding: const EdgeInsets.symmetric(horizontal: 15),
                    decoration: BoxDecoration(
                        color: const Color(0xFF1684FC),
                        borderRadius: BorderRadius.circular(5)),
                    child: const Text(
                      '选择文件',
                      style: TextStyle(color: Colors.white, fontSize: 18),
                    ),
                  ),
                  onTap: () {
                    if (_list.length == 1 &&
                        widget.uploadMode == UploadMode.SINGLE) {
                      return;
                    }
                    _selectUploadFile();
                  },
                ),
                Expanded(
                  child: Container(
                      child: ListView.builder(
                    itemBuilder: (context, index) {
                      return _buildSelectFileItem(index);
                    },
                    shrinkWrap: true,
                    itemCount: _list.length,
                  )),
                ),
                InkWell(
                  child: Container(
                    padding: const EdgeInsets.all(5),
                    margin: const EdgeInsets.symmetric(
                        horizontal: 15, vertical: 10),
                    decoration: BoxDecoration(
                        color: const Color(0xFFF4F4F4),
                        borderRadius: BorderRadius.circular(5)),
                    child: Row(
                      children: [
                        const Text(
                          '上传到：',
                          style: TextStyle(color: Colors.grey, fontSize: 14),
                        ),
                        Expanded(
                          child: Text(
                            _folderName,
                            style: const TextStyle(
                                color: Color(0xFF1684FC), fontSize: 14),
                          ),
                        ),
                        const Icon(
                          Icons.arrow_forward_ios_outlined,
                          color: Colors.grey,
                          size: 16,
                        )
                      ],
                    ),
                  ),
                  onTap: () {
                    IPFSDialog.showSelectDirDialog(context, widget.dirId,
                        (value) {
                      setState(() {
                        _folderId = value.folderId;
                        _folderName = value.folderName;
                      });
                    });
                  },
                ),
              ],
            ),
          )
        ]));
  }

  @override
  void initState() {
    super.initState();
    _folderId = widget.dirId;
    _folderName = widget.dirName;
  }

  @override
  void dispose() {
    super.dispose();
  }

  Widget _buildUploadTitle() {
    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          InkWell(
            child: const Icon(
              Icons.close,
              color: Colors.black,
              size: 25,
            ),
            onTap: () {
              Navigator.of(context).pop();
            },
          ),
          const Text(
            "选择文件",
            style: TextStyle(color: Colors.black, fontSize: 18),
          ),
          InkWell(
            child: const Text(
              '确定',
              style: TextStyle(
                  color: Color(0xFF1684FC),
                  fontSize: 18,
                  fontWeight: FontWeight.bold),
            ),
            onTap: () {
              if (_list.isNotEmpty) {
                if (widget.callback != null) {
                  widget.callback!(_list);
                }
              }
              Navigator.of(context).pop();
            },
          )
        ],
      ),
    );
  }

  Widget _buildSelectFileItem(int index) {
    UploadResultEntity entity = _list[index];
    return Container(
      padding: const EdgeInsets.only(top: 10),
      margin: const EdgeInsets.symmetric(horizontal: 15),
      child: Row(
        children: [
          IPFSUtils.getIconFont(0xe858, const Color(0xFFBDBDBD), 20.0),
          Expanded(
              child: Text(
            entity.fileName,
            style: const TextStyle(
              fontSize: 16,
              color: Colors.grey,
            ),
          )),
          InkWell(
            child: const Icon(
              Icons.close,
              color: Color(0xFFBDBDBD),
            ),
            onTap: () {
              _list.removeAt(index);
              setState(() {});
            },
          )
        ],
      ),
    );
  }

  _selectUploadFile() {
    if (widget.uploadType == UploadType.TYPE_FILE) {
      IPFSUtils.instance!.fileUpload(context, (value) {
        _list.add(value);
        setState(() {});
      }, _folderId);
    } else {
      IPFSUtils.instance!.imageUpload(context, (value) {
        _list.add(value);
        setState(() {});
      }, _folderId);
    }
  }
}
