import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/pages/file_preview_page.dart';
import 'package:vone_ipfs/pages/image_picker_dialog.dart';
import 'package:vone_ipfs/pages/online_file_page.dart';
import 'package:vone_ipfs/utils/ifps_dialog_utils.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';



class FileUploadView extends StatefulWidget {
  List<UploadResultEntity> selectedFile;
  Function(List<UploadResultEntity> files) filesCallback;
  Widget? header;
  bool? isAllowCloud;
  String dirId;
  String dirName;
  bool isFixedDir;
  FileUploadView(
      this.selectedFile, this.dirId, this.dirName, this.filesCallback,
      {Key? key, this.header, this.isAllowCloud,this.isFixedDir=false})
      : super(key: key);

  @override
  _FileUploadViewState createState() => _FileUploadViewState();
}

class _FileUploadViewState extends State<FileUploadView> {
  List<UploadResultEntity> files = [];

  @override
  void initState() {
    super.initState();
    setState(() {
      files = widget.selectedFile;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: ListView(
        shrinkWrap: true,
        children: _buildFileList(),
      ),
    );
  }

  _buildFileList() {
    List<Widget> filesWidget = [];
    filesWidget.add(InkWell(
      child: widget.header ?? _buildHeader(),
      onTap: () {
        if (widget.isAllowCloud ?? false) {
          _fileUpload();
        } else {
          _showDialog();
        }
      },
    ));
    for (int i = 0; i < files.length; i++) {
      filesWidget.add(InkWell(
        child: _buildFilesItem(i),
        onTap: () {
          Navigator.push(context, MaterialPageRoute(builder: (context) {
            return FilePreviewPage(fileUrl: files[i].filePath);
          }));
        },
      ));
    }
    return filesWidget;
  }

  _buildHeader() {
    return IntrinsicHeight(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            '上传附件',
            style: TextStyle(fontSize: 14, color: Colors.black),
          ),
          Text('上传', style: TextStyle(fontSize: 14, color: Colors.blue))
        ],
      ),
    );
  }

  _buildFilesItem(int index) {
    return Padding(
      padding: EdgeInsets.symmetric(vertical: 8),
      child: Row(
        children: [
          IPFSUtils.getFileIconBySuffix(files[index].fileName),
          SizedBox(
            width: 10,
          ),
          Expanded(
            child: Text(
              '${files[index].filePath}',
              overflow: TextOverflow.ellipsis,
              style: TextStyle(fontSize: 14, color: Colors.black87),
            ),
          ),
          InkWell(
            child: Icon(
              Icons.close,
              size: 20,
            ),
            onTap: () {
              files.removeAt(index);
              if (widget.filesCallback != null) {
                widget.filesCallback(files);
              }
              setState(() {});
            },
          )
        ],
      ),
    );
  }

  _showDialog() {
    showModalBottomSheet(
        backgroundColor: Colors.transparent,
        context: context, //state.context,
        builder: (BuildContext context) {
          return ImagePickerDialog(
            localCallback: () {
              if(widget.isFixedDir){
                _fileUpload();
              }else{
                IPFSDialog.showFileUpload(context, widget.dirId,
                    widget.dirName, (value) {
                      setState(() {
                        files.addAll(value);
                      });
                    });
              }

            },
            cloudCallback: () {
              Navigator.push(context, MaterialPageRoute(builder: (context) {
                return OnlineFilePage(widget.dirId,widget.dirName,(result) {
                  if (result.isNotEmpty) {
                    for (var element in result) {
                      String filePath = IPFSUtils.instance!.getFilePath(
                          element.fileID, element.fileMeta.fileName);
                      String coverPath = '';
                      files.add(UploadResultEntity(
                          filePath, coverPath, element.fileMeta.fileName));
                    }
                    if (widget.filesCallback != null) {
                      widget.filesCallback(files);
                    }
                    setState(() {});
                  }
                });
              }));
            },
          );
        });
  }

  _fileUpload() {
    IPFSUtils.instance!.fileUpload(context, (value) {
      files.add(value);
      if (widget.filesCallback != null) {
        widget.filesCallback(files);
      }
      setState(() {});
    }, widget.dirId);
  }
}
