import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/folder_data_entity.dart';
import 'package:vone_ipfs/bean/select_folder_entity.dart';
import 'package:vone_ipfs/generated/json/base/json_convert_content.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/component/create_folder_view.dart';

class SelectDirView extends StatefulWidget {
  final ValueChanged<SelectFolderEntity> callback;
  String dirId;
  SelectDirView(
      this.dirId,
    this.callback, {
    Key? key,
  }) : super(key: key);

  @override
  SelectDirViewState createState() => new SelectDirViewState();
}

class SelectDirViewState extends State<SelectDirView> {
  List<FolderDataDataFileList> _list = [];
  String default_id = '';
  String _parentId = "";
  String _selectId = '';
  String _selectName = '';

  _dismissDialog() {
    Navigator.of(context).pop();
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      type: MaterialType.transparency,
      child: Column(
        children: <Widget>[
          Expanded(
            child: GestureDetector(
              behavior: HitTestBehavior.translucent,
              onTap: _dismissDialog,
              child: Container(),
            ),
          ),
          Container(
            decoration: const BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(10.0),
                  topRight: Radius.circular(10.0)),
            ),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _buildTitle(),
                const Divider(
                  color: Color(0xFFF1F1F6),
                  height: 1,
                ),
                Container(
                  height: MediaQuery.of(context).size.height * 0.75,
                  child: _list.isNotEmpty
                      ? ListView(
                          children: _buildDir(),
                          shrinkWrap: true,
                        )
                      : _buildNullFolder(),
                ),
                Container(
                  padding: EdgeInsets.all(20),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      InkWell(
                        child: Row(
                          children: const [
                            Icon(
                              Icons.create_new_folder,
                              color: Color(0xFF1684FC),
                              size: 30,
                            ),
                            SizedBox(
                              width: 10,
                            ),
                            Text(
                              '新建文件夹',
                              style: TextStyle(
                                  color: Color(0xFF1684FC), fontSize: 16),
                            )
                          ],
                        ),
                        onTap: () {
                          _showCreateFolerDialog();
                        },
                      ),
                      InkWell(
                        child: Container(
                          height: 40,
                          padding: const EdgeInsets.symmetric(horizontal: 15),
                          alignment: Alignment.center,
                          decoration: BoxDecoration(
                              color: const Color(0xFF1684FC),
                              borderRadius: BorderRadius.circular(40)),
                          child: const Text(
                            '上传到此',
                            style: TextStyle(color: Colors.white, fontSize: 16),
                          ),
                        ),
                        onTap: () {
                          print(_selectId);
                          print(_selectName);
                          widget.callback(
                              SelectFolderEntity(_selectId, _selectName));
                          Navigator.of(context).pop();
                        },
                      )
                    ],
                  ),
                )
                // ListView.builder(
                //   itemBuilder: (context, index) {
                //     return _buildDirItem(index);
                //   },
                //   shrinkWrap: true,
                // ),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildTitle() {
    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Visibility(
            child: InkWell(
              child: const Text(
                '返回',
                style: TextStyle(fontSize: 16, color: Color(0xFF1684FC)),
              ),
              onTap: () {
                _getFolderData(_parentId);
              },
            ),
          ),
          const Text(
            "选择文件夹",
            style: TextStyle(color: Colors.black, fontSize: 18),
          ),
          InkWell(
            child: const Icon(
              Icons.close,
              color: Colors.black,
              size: 25,
            ),
            onTap: () {
              Navigator.of(context).pop();
            },
          )
        ],
      ),
    );
  }

  Widget _buildNullFolder() {
    return Center(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          IPFSUtils.getIconFont(
            0xe622,
            const Color(0xFFFFD23D),
            60.0,
          ),
          const SizedBox(
            height: 10,
          ),
          Text(
            '这里是空的',
            style:
                TextStyle(fontSize: 14, color: Colors.black.withOpacity(0.6)),
          )
        ],
      ),
    );
  }

  List<Widget> _buildDir() {
    List<Widget> list = [];
    for (int i = 0; i < _list.length; i++) {
      list.add(_buildDirItem(i));
    }
    return list;
  }

  Widget _buildDirItem(int index) {
    return InkWell(
      child: Container(
        margin: const EdgeInsets.only(left: 20, top: 20),
        child: Row(
          children: [
            const Icon(
              IconData(0xe622,
                  fontFamily: 'iconfont', fontPackage: 'vone_ipfs'),
              color: Color(0xFFFFD23D),
              size: 25,
            ),
            const SizedBox(
              width: 20,
            ),
            Expanded(
                child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  _list[index].fileMeta?.fileName ?? '未知',
                  style: const TextStyle(fontSize: 16, color: Colors.black87),
                ),
                Text(
                  _list[index].fileMeta?.uploadTime ?? '未知',
                  style: const TextStyle(fontSize: 14, color: Colors.grey),
                ),
              ],
            ))
          ],
        ),
      ),
      onTap: () {
        setState(() {
          _selectName = _list[index].fileMeta!.fileName!;
          _selectId = _list[index].fileID.toString();
        });
        _getFolderData(_selectId);
      },
    );
  }

  @override
  void initState() {
    super.initState();
    default_id = widget.dirId;
    _parentId = widget.dirId;
    _getFolderData(default_id);
  }

  @override
  void dispose() {
    super.dispose();
  }

  ///显示创建文件夹的dialog
  _showCreateFolerDialog() {
    showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        isScrollControlled: true,
        builder: (BuildContext context) {
          return CreateFolderView(_selectId, () {
            _getFolderData(_selectId);
          });
        });
  }
  ///获取文件夹列表
  _getFolderData(parentId) async {
    print(parentId);
    Response response;
    Dio dio = Dio();
    Map<String, dynamic> param = {
      "UserID": IPFSUtils.instance!.userId,
      "OnlyCatalogue": '1',
      "SecretKey": IPFSUtils.instance!.secretKey,
      "ParentID": parentId
    };
    response =
        await dio.get<String>(HttpApi.getFolderList, queryParameters: param);
    print(response.data.toString());
    if (200 == response.statusCode) {
      FolderDataEntity fileData = JsonConvert.fromJsonAsT<FolderDataEntity>(
          jsonDecode(response.data.toString()));
      if (fileData.code == 0) {
          if(fileData.data!=null){
            _parentId = fileData.data!.parentID.toString();
            if(_parentId.isEmpty || _parentId=='null'){
              _parentId = default_id;
            }
            setState(() {
            });
          }else{
            _parentId = default_id;
          }
        if (fileData.data!.fileList != null &&
            fileData.data!.fileList!.isNotEmpty) {
          setState(() {
            _selectId = parentId;
            _list = fileData.data!.fileList!;
          });
        } else {
          setState(() {
            _list = [];
            _selectId = parentId;
          });
        }
      } else {
        setState(() {
          _list = [];
        });
      }
    } else {
      setState(() {
        _list = [];
      });
    }
  }
}
