import 'package:flutter/material.dart';

class VoneImage extends StatelessWidget {
  ///图片宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///默认图片
  String? placeholder;

  ///图片地址
  final String imageUrl;

  ///图片形状
  ImageShape shape;

  ///圆角大小
  double radiusSize;

  final BoxFit boxFit;

  VoneImage(
      {Key? key,
      required this.imageWidth,
      required this.imageHeight,
      this.placeholder,
      required this.imageUrl,
      this.shape = ImageShape.None,
      this.boxFit = BoxFit.cover,
      this.radiusSize = 0})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    placeholder ?? 'ic_default_avatar.png';
    Widget child;
    final isNetwork = imageUrl.startsWith(RegExp(r'^http'));
    if (isNetwork) {
      if (imageUrl.endsWith("null") || imageUrl.endsWith("/")) {
        child = Image.asset(
          placeholder!,
          width: imageWidth,
          height: imageHeight,
          fit: boxFit,
        );
      } else {
        child = FadeInImage.assetNetwork(
            placeholder: 'bg_img.png',
            image: imageUrl,
            width: imageWidth,
            height: imageHeight,
            fit: boxFit,
            imageErrorBuilder: (context, error, _) {
              return Image.asset(
                placeholder!,
                width: imageWidth,
                height: imageHeight,
                fit: boxFit,
              );
            });
      }
    } else {
      if (imageUrl.isEmpty || imageUrl == 'null') {
        child = Image.asset(
          placeholder!,
          fit: boxFit,
          width: imageWidth,
          height: imageHeight,
        );
      } else {
        child = Image.asset(
          imageUrl,
          fit: boxFit,
          width: imageWidth,
          height: imageHeight,
        );
      }
    }
    if (shape == ImageShape.FILL) {
      return ClipRRect(
        borderRadius: BorderRadius.circular(radiusSize), // 圆角半径
        child: Container(
          width: imageWidth,
          height: imageHeight,
          child: child,
        ),
      );
    } else if (shape == ImageShape.ROUND) {
      return ClipOval(
        child: Container(
          width: imageWidth,
          height: imageHeight,
          child: child,
        ),
      );
    } else {
      return child;
    }
  }
}

enum ImageShape {
  ///原图
  None,

  ///圆角
  FILL,

  ///圆形
  ROUND
}

